import pygame
import os
import screen
from Numeric import *
from pygame.locals import *

# Factor used to scale the images in the screen
SCREEN_FACTOR = 1

# Allows to run the game using the framework
class Game:    
    # Constructor
    # - title: Game's title.    
    def __init__(self, title):
        # Initialize pygame                    
        pygame.init()
        
        # Create the game window
        pygame.display.init()
        self.window = pygame.display.set_mode((int(600 * SCREEN_FACTOR), int(450 * SCREEN_FACTOR)))        
        
        # Set title
        pygame.display.set_caption(title)
                
        # Set initial state
        self.show_fps = True
        self.fps = 0    
        self.quit_game = 0
        self.font = pygame.font.Font('fonts/freesansbold.ttf', 13)
        
    # Runs the game
    # - initial_screen: Screen that is shown when the game starts
    def run(self, initial_screen):
        self.screen = initial_screen
        
        # Main loop
        frames_duration = 0
        frames_count = 0                
        while not self.quit_game:            
            # Do operations per tick
            before_tick = pygame.time.get_ticks()
            self.screen.notify_tick()
            after_tick = pygame.time.get_ticks()
            
            # Make sure game doesn't run at more than 40 frames per second. This
            # avoid that the use of CPU goes up to 100%
            duration = after_tick - before_tick
            if duration < 1000 / 40:
                pygame.time.delay(1000 / 40 - duration)
            
            # Calculate frames per second (FPS)
            frames_duration += duration
            frames_count += 1
            if frames_count >= 40:
                if frames_duration == 0:
                    self.fps = 50000
                else:
                    self.fps = 1000 * 40 / frames_duration
                frames_count = 0
                frames_duration = 0
            
        
    # Ends the execution of the game
    def quit(self):
        # Stops the main loop
        self.quit_game = 1
        
    # Loads the specified image
    # - file_name: File name    
    def load_image(self, file_name):
        full_name = os.path.join('images', file_name)
        try:
            image = pygame.image.load(full_name)
        except pygame.error, message:
            print 'Cannot load image:', full_name
            raise SystemExit, message

        # Convert the image to the same pixel format as the screen.
        # This is the fastest format to blitting        
        if image.get_bitsize() == 32:            
            image = image.convert_alpha()
        else:
            image = image.convert()
        
        # Adjust image size applying the SCREEN_FACTOR
        image = self.__adjust_surface_size(image)
        
        return screen.Image(image)                

    # Loads the specified image with a separated alpha mask
    # - file_name_rgb: File name (with RGB colors)   
    # - file_name_alpha: File name (with alpha channel)
    def load_image_alpha(self, file_name_rgb, file_name_alpha):
        full_name = os.path.join('images', file_name_rgb)
        try:
            image_rgb = pygame.image.load(full_name)            
        except pygame.error, message:
            print 'Cannot load image:', full_name
            raise SystemExit, message

        full_name = os.path.join('images', file_name_alpha)
        try:
            image_alpha = pygame.image.load(full_name)            
        except pygame.error, message:
            print 'Cannot load image:', full_name
            raise SystemExit, message
        
        # Merge image RGB with alpha channel
        image = image_rgb.convert_alpha()
        alpha = pygame.surfarray.pixels_alpha(image);                
        for x in range(image.get_width()):
            for y in range(image.get_height()):                                
                alpha[x][y] = image_alpha.get_at((x, y))[0]        

        # Adjust image size applying the SCREEN_FACTOR
        image = self.__adjust_surface_size(image)
        
        return screen.Image(image)   
        
    # Gets a value that indicates if the FPS is visible
    def get_show_fps(self):
        return self.show_fps
            
    # Sets a value that indicates if the FPS is visible
    def set_show_fps(self, value):        
        self.show_fps = value
        self.screen.background_dirty = True
    
    # Draw the FPS in the screen
    # - dirty_rects: List where the rect that must be redrawn is appended
    def draw_fps(self, dirty_rects):        
        # Render the text
        text = self.font.render("FPS: " + str(round(self.fps, 2)), True, (0, 0, 0))
        
        # Show the text in the screen
        text_rect = text.get_rect()
        text_rect.topleft = (4, 2)
        border_rect = Rect(0, 0, 90, text_rect.height + 4)        
        pygame.draw.rect(self.window, (250, 250, 250), border_rect)                
        pygame.draw.rect(self.window, (20, 20, 20), border_rect, 1)
        self.window.blit(text, text_rect)  
        dirty_rects.append(border_rect)                    
        
    # Adjust the surface size applying the screen factor
    # - surface: Source surface    
    def __adjust_surface_size(self, surface):                        
        if SCREEN_FACTOR == 2:
            # Increase the image size x 2
            return pygame.transform.scale2x(surface)
        elif SCREEN_FACTOR != 1:
            # Increase the image size x 2
            return pygame.transform.scale(surface, (int(surface.get_width() * SCREEN_FACTOR), 
                int(surface.get_height() * SCREEN_FACTOR)))
        else:
            return surface
