import framework.screen
from framework.screen import ScreenBase, ScreenObjectImage, ScreenObjectCustomDraw, ScreenObjectCell
import pygame
from pygame.locals import *

# Defines the screen of phase 2
class Screen(ScreenBase):
    def __init__(self, game):                
        ScreenBase.__init__(self, game, 1, game.load_image("p2-background-small.jpg"))
        self.game = game
        
        # Load images
        #self.magnifier_image = game.load_image("p2-magnifier.png")
        self.magnifier_image = game.load_image_alpha("p2-magnifier-rgb.gif", "p2-magnifier-alpha.gif")
        self.magnifier_mask = game.load_image("p2-magnifier-mask.gif")
        self.background_large = game.load_image("p2-background-large.jpg")        
        
        # Create screen objects
        magnifier_width = self.magnifier_image.get_width()
        magnifier_height = self.magnifier_image.get_height()
        self.magnifier = ScreenObjectCustomDraw(self, 390, 320, magnifier_width, magnifier_height, self.draw_magnifier)
        
        # Add the objects in the layers        
        self.add_object(0, self.magnifier)
                                        
        # Set initial state
        self.magnifier_taken = False
        
        
        # --- Examples ---
        
        # Load two magnifieres
        #self.magnifier_2 = ScreenObjectImage(self, 10, 320, self.magnifier_image)
        #self.add_object(0, self.magnifier_2)
        #self.magnifier_3 = ScreenObjectImage(self, 50, 320, self.magnifier_image)
        #self.add_object(0, self.magnifier_3)
        
        # Mosaic
        #self.mosaic_image = game.load_image("alien.gif")
        #self.mosaic = ScreenObjectImage(self, 200, 380, self.mosaic_image)
        #self.add_object(0, self.mosaic)
        #self.mosaic_cell_width = 20
        #self.mosaic_cell_height = 20        
        #self.mosaic_item = ScreenObjectCell(self, 280, 380, self.mosaic_image, self.mosaic_cell_width, self.mosaic_cell_height)
        #self.mosaic_item.set_cell(1, 1)
        #self.add_object(0, self.mosaic_item)                


    # Draws the magnifier in the screen
    # - screen: Target screen.
    # - surface: Target surface.
    # - draw_rect: Rectangle where the mangifier is shown
    def draw_magnifier(self, screen, surface, draw_rect):
        # Draws the amplified background 
        x = draw_rect.left
        y = draw_rect.top
        back_x = draw_rect.left * 2 + 73
        back_y = draw_rect.top * 2 + 70
        delta_x = 27
        delta_y = 16
        screen.blit(surface, self.background_large, (x + delta_x, y + delta_y), 
            Rect(back_x + delta_x, back_y + delta_y, 89, 20))
        delta_x = 15
        delta_y = 36
        screen.blit(surface, self.background_large, (x + delta_x, y + delta_y), 
            Rect(back_x + delta_x, back_y + delta_y, 113, 70))
        delta_x = 27
        delta_y = 106
        screen.blit(surface, self.background_large, (x + delta_x, y + delta_y), 
            Rect(back_x + delta_x, back_y + delta_y, 89, 21))
        
        # Draws the magnifier
        screen.blit(surface, self.magnifier_image, draw_rect)
                
    # Handle an event in the screen
    # - e: Event.    
    def handle_event(self, e):
        if e.type == MOUSEBUTTONDOWN:
            if e.button == 1:                                            
                x, y = e.pos
                
                # if self.mosaic.is_over_rect(x, y):
                #    row, column = self.mosaic.get_over_cell(x, y, self.mosaic_cell_width, self.mosaic_cell_height)                    
                #    self.mosaic_item.set_cell(row, column)
                
                if self.magnifier_taken:
                    # Drop the magnifier
                    self.set_mouse_cursor(None)
                    pygame.event.set_grab(False)       
                    self.magnifier_taken = False
                else:
                    # Check is the position is over the rectangle that contains
                    # the magnifier
                    if self.magnifier.is_over_rect(x, y):
                        # Check position in mask to detect if it inside the magnifier                    
                        c = self.magnifier_mask.get_at(x - self.magnifier.get_left(), y - self.magnifier.get_top())
                
                        if c[0] == 0:
                            # Take the magnifier                        
                            self.set_mouse_cursor(self.magnifier, False, Rect(-20, -20, 762, 600))
                            pygame.event.set_grab(True)
                            self.magnifier_taken = True            
        elif e.type == KEYDOWN:
            if e.key == K_ESCAPE:
                self.game.quit()
            elif e.key == K_RIGHT:
                self.magnifier.left += 20
            elif e.key == K_LEFT:
                self.magnifier.left -= 20
            elif e.key == K_DOWN:
                self.magnifier.top += 20
            elif e.key == K_UP:
                self.magnifier.top -= 20
                                 